@@ -27,45 +27,57 @@
 #include "SDL_cocoashape.h"
 #include "../SDL_sysvideo.h"
 
+@implementation SDL_ShapeData
+@end
+
+@interface SDL_CocoaClosure : NSObject
+    @property (nonatomic) NSView* view;
+    @property (nonatomic) NSBezierPath* path;
+    @property (nonatomic) SDL_Window* window;
+@end
+
+@implementation SDL_CocoaClosure
+@end
+
 SDL_WindowShaper*
 Cocoa_CreateShaper(SDL_Window* window)
+{ @autoreleasepool
 {
-    SDL_WindowData* windata = (SDL_WindowData*)window->driverdata;
-    [windata->nswindow setOpaque:NO];
+    SDL_WindowShaper* result;
+    SDL_ShapeData* data;
+    int resized_properly;
+    SDL_WindowData* windata = (__bridge SDL_WindowData*)window->driverdata;
+    [windata.nswindow setOpaque:NO];
 
-    [windata->nswindow setStyleMask:NSWindowStyleMaskBorderless];
+    [windata.nswindow setStyleMask:NSWindowStyleMaskBorderless];
 
-    SDL_WindowShaper* result = (SDL_WindowShaper *)SDL_malloc(sizeof(SDL_WindowShaper));
+    result = (SDL_WindowShaper *)SDL_malloc(sizeof(SDL_WindowShaper));
     result->window = window;
     result->mode.mode = ShapeModeDefault;
     result->mode.parameters.binarizationCutoff = 1;
     result->userx = result->usery = 0;
     window->shaper = result;
 
-    SDL_ShapeData* data = (SDL_ShapeData *)SDL_malloc(sizeof(SDL_ShapeData));
-    result->driverdata = data;
-    data->context = [windata->nswindow graphicsContext];
-    data->saved = SDL_FALSE;
-    data->shape = NULL;
+    data = [[SDL_ShapeData alloc] init];
+    data.context = [windata.nswindow graphicsContext];
+    data.saved = SDL_FALSE;
+    data.shape = NULL;
 
-    int resized_properly = Cocoa_ResizeWindowShape(window);
+    /* TODO: There's no place to release this... */
+    result->driverdata = (void*) CFBridgingRetain(data);
+
+    resized_properly = Cocoa_ResizeWindowShape(window);
     SDL_assert(resized_properly == 0);
     return result;
-}
-
-typedef struct {
-    NSView* view;
-    NSBezierPath* path;
-    SDL_Window* window;
-} SDL_CocoaClosure;
+}}
 
 void
 ConvertRects(SDL_ShapeTree* tree, void* closure)
 {
-    SDL_CocoaClosure* data = (SDL_CocoaClosure*)closure;
+    SDL_CocoaClosure* data = (__bridge SDL_CocoaClosure*)closure;
     if(tree->kind == OpaqueShape) {
-        NSRect rect = NSMakeRect(tree->data.shape.x,data->window->h - tree->data.shape.y,tree->data.shape.w,tree->data.shape.h);
-        [data->path appendBezierPathWithRect:[data->view convertRect:rect toView:nil]];
+        NSRect rect = NSMakeRect(tree->data.shape.x, data.window->h - tree->data.shape.y, tree->data.shape.w, tree->data.shape.h);
+        [data.path appendBezierPathWithRect:[data.view convertRect:rect toView:nil]];
     }
 }
 
@@ -73,26 +85,28 @@ int
 Cocoa_SetWindowShape(SDL_WindowShaper *shaper, SDL_Surface *shape, SDL_WindowShapeMode *shape_mode)
 { @autoreleasepool
 {
-    SDL_ShapeData* data = (SDL_ShapeData*)shaper->driverdata;
-    SDL_WindowData* windata = (SDL_WindowData*)shaper->window->driverdata;
-    SDL_CocoaClosure closure;
-    if(data->saved == SDL_TRUE) {
-        [data->context restoreGraphicsState];
-        data->saved = SDL_FALSE;
+    SDL_ShapeData* data = (__bridge SDL_ShapeData*)shaper->driverdata;
+    SDL_WindowData* windata = (__bridge SDL_WindowData*)shaper->window->driverdata;
+    SDL_CocoaClosure* closure;
+    if(data.saved == SDL_TRUE) {
+        [data.context restoreGraphicsState];
+        data.saved = SDL_FALSE;
     }
 
-    /*[data->context saveGraphicsState];*/
-    /*data->saved = SDL_TRUE;*/
-    [NSGraphicsContext setCurrentContext:data->context];
+    /*[data.context saveGraphicsState];*/
+    /*data.saved = SDL_TRUE;*/
+    [NSGraphicsContext setCurrentContext:data.context];
 
     [[NSColor clearColor] set];
-    NSRectFill([windata->sdlContentView frame]);
-    data->shape = SDL_CalculateShapeTree(*shape_mode,shape);
+    NSRectFill([windata.sdlContentView frame]);
+    data.shape = SDL_CalculateShapeTree(*shape_mode, shape);
 
-    closure.view = windata->sdlContentView;
+    closure = [[SDL_CocoaClosure alloc] init];
+
+    closure.view = windata.sdlContentView;
     closure.path = [NSBezierPath bezierPath];
     closure.window = shaper->window;
-    SDL_TraverseShapeTree(data->shape,&ConvertRects,&closure);
+    SDL_TraverseShapeTree(data.shape, &ConvertRects, (__bridge void*)closure);
     [closure.path addClip];
 
     return 0;
@@ -100,11 +114,11 @@ Cocoa_SetWindowShape(SDL_WindowShaper *shaper, SDL_Surface *shape, SDL_WindowSha
 
 int
 Cocoa_ResizeWindowShape(SDL_Window *window)
-{
-    SDL_ShapeData* data = window->shaper->driverdata;
+{ @autoreleasepool {
+    SDL_ShapeData* data = (__bridge SDL_ShapeData*)window->shaper->driverdata;
     SDL_assert(data != NULL);
     return 0;
-}
+}}
 
 #endif /* SDL_VIDEO_DRIVER_COCOA */
 
